local _, RS = ...
RS.Messages = {}
local Messages = RS.Messages

-- local references - WoW and Lua
local table_concat = table.concat
local table_insert = table.insert
local string_upper = string.upper
local string_gmatch = string.gmatch
local string_rep = string.rep

-- local references - Addon
local AttemptReaction = nil
local Variables = nil

function Messages:OnLoad()
	AttemptReaction = RS.Reactions.AttemptReaction
	Variables = RS.Variables
end

----------------------------------------------------------------------------------------------------
-- helper functions
----------------------------------------------------------------------------------------------------
-- add a ' or 's to the end of a name
local function ApostropheName(name)
	return name and (name..(name:sub(-1) == "s" and "'" or "'s")) or ""
end

-- Word wrapping - barely modified version from http://rosettacode.org/wiki/Word_wrap#Lua
local function SplitTokens(text)
	local res = {}
	for word in string_gmatch(text, "%S+") do
		res[#res+1] = word
	end
	return res
end

local function WordWrap(text, linewidth)
	if not linewidth then
		linewidth = 250
	end

	-- replace all spaces in text inside [ ] with a character no item/spell/quest uses
	local changedLink = false
	text = text:gsub("(%[.-%])", function(link) changedLink = true; return link:gsub(" ", "@") end)

	local spaceleft = linewidth
	local res = {}
	local line = {}
	for _, word in ipairs(SplitTokens(text)) do
		if #word + 1 > spaceleft then
			table_insert(res, table_concat(line, ' '))
			line = {word}
			spaceleft = linewidth - #word
		else
			table_insert(line, word)
			spaceleft = spaceleft - (#word + 1)
		end
	end
	table_insert(res, table_concat(line, ' '))

	-- convert links back to using spaces
	if changedLink then
		for i=1,#res do
			res[i] = res[i]:gsub("(%[.-%])", function(link) return link:gsub("@", " ") end)
		end
	end
	return res
end

----------------------------------------------------------------------------------------------------
-- event and message variables
----------------------------------------------------------------------------------------------------
-- list of special cases for when deciding to use "a" or "an" - ListA will be checked first, then
-- listAn, then if neither match it's assumed "a" is the right choice
local ArticleListA = {
	"^on[c]?e.*",
	"^unicorn.*",
	"^union.*",
	"^unit.*",
	"^univ.*",
	"^us[a]?$",
	"^use.*",
	"^util.*",
	"^euro.*",
	"^utopi.*",
	"^herbi.*", -- so the herb* in "an" doesn't match herbivore
	"^us[a]?$",
	"^u[k]?$",
	"^ufo$",
}
local ArticleListAn = {
	"^[aeiou].*",
	"^heir.*",
	"^herb.*",     -- Europeans may need to remove this one
 --"^historic.*", -- you can uncomment this if you're an old person
	"^honest.*",
	"^hono[u]?r.*",
	"^hour.*",
	"^8.*",
	"^1[18]$",
	"^mvp$",
	"^[fhlmnrsx]$", -- single letters, like "Pierre gets an F for that attack"
}

----------------------------------------------------------------------------------------------------
-- send a message
----------------------------------------------------------------------------------------------------
-- used to execute slash commands
local MacroEditBox = MacroEditBox
local MacroEditBox_OnEvent = MacroEditBox:GetScript("OnEvent")

-- to know if they're sending a whisper so long messages can be split up properly
local whisperCommands = {}
do
	local i = 1
	local command
	repeat
		command = _G["SLASH_WHISPER"..i]
		if command then
			whisperCommands[command] = 1
			i = i + 1
		end
	until not command
end

-- for knowing when to convert a message to use emotes instead
local emoteModeCommands = {}
do
	local command
	local i = 1
	repeat
		command = _G["SLASH_SAY"..i]
		if command then
			emoteModeCommands[command] = 1
			i = i + 1
		end
	until not command

	i = 1
	repeat
		command = _G["SLASH_YELL"..i]
		if command then
			emoteModeCommands[command] = 1
			i = i + 1
		end
	until not command
end

-- for faster checking when stealthed and wanting to be quiet
local quietCommands = {[SLASH_SAY1]=1, [SLASH_YELL1]=1, [SLASH_EMOTE1]=1}
do
	local command
	local i, j

	i = 1
	repeat
		command = _G["SLASH_SAY"..i]
		if command then
			quietCommands[command] = 1
			i = i + 1
		end
	until not command

	i = 1
	repeat
		command = _G["SLASH_YELL"..i]
		if command then
			quietCommands[command] = 1
			i = i + 1
		end
	until not command

	i = 1
	local emoteGeneric, emoteCommand
	repeat
		emoteGeneric = _G["EMOTE"..i.."_TOKEN"] -- generic name like BYE that can be used by multiple commands
		if emoteGeneric then
			j=1
			repeat
				emoteCommand = _G["EMOTE"..i.."_CMD"..j] -- each command for the generic name, like /goodbye and /farewell
				if emoteCommand then
					quietCommands[emoteCommand] = 1
					j = j + 1
				end
			until not emoteCommand
			i = i + 1
		end
	until not emoteGeneric
end

function Messages.UseMessage(message, channel, language)
	local addonSettings = ReactionsSave

	if channel == "force_message" then
		DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Test message:|r " .. message)
		return
	end

	if addonSettings.testing then
		DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reaction:|r " .. channel .. "" .. (target and (" " .. target) or "") .. ": " .. message)
		return
	end

	if channel == "chat command" then
		if message:sub(1,1) ~= "/" then
			message = SLASH_SAY1.." " .. message -- have to add the /s or sticky channels may make them use something else
		elseif message:match("/%d+") then
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reactions:|r Addons can no longer send automatic messages on numbered channels, so can't use: " .. message, 1, 1, 0)
			return
		end

		local command = message:match("^(%S+)"):lower()
		if IsSecureCmd(command) then
			DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reactions:|r The slash command \"" .. message .. "\" can't be used automatically.", 1, 1, 0)
			return
		end

		local inInstance, instanceType = IsInInstance()
		if IsStealthed() and addonSettings.quietStealth and quietCommands[command] then
			if not inInstance or instanceType == "pvp" or instanceType == "arena" then
				return
			end
		end

		local fullCommand

		if not inInstance and emoteModeCommands[command] then
			if not addonSettings.emoteModeWarning then
				DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reactions:|r It's no longer possible to automatically say or yell outside instances. In the options, if \"World emote mode\" is on it will change blocked say/yell speech to emotes. This message won't be shown again!", 1, 0, 0)
				addonSettings.emoteModeWarning = true
			end
			if not addonSettings.emoteMode then
				return
			end
			fullCommand = command == SLASH_SAY1 and "/e says: " or "/e yells: "
			command = "/e"
			message = message:gsub("^%S+ ", fullCommand)
		end

		-- switch to the desired language if needed
		local previousLanguage
		if language then
			-- set to false if none exists so that it can be erased later
			previousLanguage = MacroEditBox.language or false
			MacroEditBox.language = language
		end

		-- get the full command including a possible whisper target and trailing space
		if not fullCommand then
			fullCommand = whisperCommands[command] and message:match("^(/%S+%s+%S+ )") or (command.." ")
		end

		-- finally use the command, splitting it up if needed
		if #message - #fullCommand <= 255 then
			MacroEditBox_OnEvent(MacroEditBox, "EXECUTE_CHAT_LINE", message)
		else
			-- must remove the command before splitting, then add it back each line
			local splitMessage = WordWrap((message:match("^"..fullCommand.."(.+)")), 255)
			for i=1,#splitMessage do
				MacroEditBox_OnEvent(MacroEditBox, "EXECUTE_CHAT_LINE", fullCommand..splitMessage[i])
			end
		end

		-- set the language back to what it was
		if previousLanguage ~= nil then
			MacroEditBox.language = previousLanguage or nil
		end
		return
	end

	if channel == "script" then
		RunScript(message)
		return
	end

	if channel == "print to chat" then
		DEFAULT_CHAT_FRAME:AddMessage(message)
		return
	end

	if channel == "print to warning" then
		RaidNotice_AddMessage(RaidWarningFrame, message, ChatTypeInfo["RAID_WARNING"])
		PlaySoundFile(567397) -- sound/interface/raidwarning.ogg
		return
	end
end

----------------------------------------------------------------------------------------------------
-- parse a message
----------------------------------------------------------------------------------------------------
local raidTargetIconList = {
	["star"]=1, ["circle"]=1, ["diamond"]=1, ["triangle"]=1, ["moon"]=1, ["square"]=1, ["cross"]=1, ["skull"]=1,
	["rt1"]=1, ["rt2"]=1, ["rt3"]=1, ["rt4"]=1, ["rt5"]=1, ["rt6"]=1, ["rt7"]=1, ["rt8"]=1,
	-- in case they're not using an english client:
	[RAID_TARGET_1]=1, [RAID_TARGET_2]=1, [RAID_TARGET_3]=1, [RAID_TARGET_4]=1, [RAID_TARGET_5]=1,
	[RAID_TARGET_6]=1, [RAID_TARGET_7]=1, [RAID_TARGET_8]=1,
}

local capitalizeChannels = {
	[SLASH_BATTLEGROUND1]=1,
	[SLASH_GUILD1]=1, [SLASH_OFFICER1]=1, [SLASH_PARTY1]=1, [SLASH_RAID1]=1,
	[SLASH_RAID_WARNING1]=1, [SLASH_SAY1]=1, [SLASH_WHISPER1]=1, [SLASH_YELL1]=1,
}

local function ExpandOptions(text)
	text = text:gsub("(%b())", function(fullStr)
		local str = fullStr:sub(2, -2)

		if str:find("%(") then
			str = ExpandOptions(str)
		end

		local _, amount = str:gsub("|", "|")
		if amount == 0 then
			return fullStr
		else
			local choice = random(0, amount)
			local pattern = string_rep("[^|]+.-|", choice) .. "([^|]+)"
			return (str:match(pattern)) or ""
		end
	end)
	return text
end

function Messages.ParseAndSendMessage(text, channel, target, language, reactionData, addTime)
	channel = channel:lower()

	-- convert <new> variables into a special pattern to make it easy to iterate through lines:
	-- \1\2time\2channel\2
	text = text:gsub("<new%s*(%d*)%s*(%d*)[:]*(.-)>", function(time1, time2, channel)
		if time2 ~= "" then
			time1 = random(tonumber(time1) or 1, tonumber(time2) or 1)
		else
			time1 = time1 ~= "" and time1 or "0"
		end
		-- "chat" is short for "chat command"
		return "\1\2" .. time1 .. "\2" .. (channel == "chat" and "chat command" or channel) .. "\2"
	end)

	-- go through each line separated by \1
	local lineTime = addTime or 0
	for line in string_gmatch(text, "[^\1]+") do
	repeat -- to be able to use break as continue
		local lineChannel = nil
		-- if it's a <new> line, get the time and channel
		line = line:gsub("^\2(%d+)\2(.-)\2", function(time, channel)
			lineTime = lineTime + tonumber(time)
			lineChannel = channel
			return ""
		end)
		if line == "" then
			break -- acts as continue
		end
		if channel == "force_message" then
			lineChannel = "force_message"
		else
			lineChannel = lineChannel ~= "" and lineChannel or channel
		end

		--------------------------------------------------
		-- scripts
		--------------------------------------------------
		if lineChannel == "chat command" and (line:find("^"..SLASH_SCRIPT1) or line:find("^"..SLASH_SCRIPT2)) then
			-- only replace normal variables in scripts
			line = line:gsub("<(.-)>", function(variable)
				local apostropheCheck
				if variable:sub(-2) == "'s" then
					variable = variable:sub(1, -3)
					apostropheCheck = true
				end
				local name, arg = variable:match("([%w_]+)[:]*(.*)")
				local text = Variables:Lookup(name, arg) or "<"..variable..">"
				return apostropheCheck and ApostropheName(text) or text
			end)

			if lineTime > 0 then
				RS.Queue:AddFutureReaction(line, lineChannel, nil, lineTime)
			else
				Messages.UseMessage(line, lineChannel)
			end
			break -- acts as continue
		end

		--------------------------------------------------
		-- non-script channels
		--------------------------------------------------
		-- replace {} tags
		local tagList = ReactionsSave.tagList
		line = line:gsub("{(.-)}", function(tags)
			if raidTargetIconList[tags] then -- don't replace raid target icons
				return "{"..tags.."}"
			end
			local apostropheCheck
			if tags:sub(-2) == "'s" then
				tags = tags:sub(1, -3)
				apostropheCheck = true
			end
			local list = {}
			local total = 0
			for name in string_gmatch(tags, "([^|]+)") do
				if tagList[name] then
					list[#list+1] = tagList[name]
					total = total + tagList[name].amount
				else
					DEFAULT_CHAT_FRAME:AddMessage("Reactions: Unknown tag '" .. name .. "' tried to be used!", 1, 0, 0)
				end
			end
			if total == 0 then
				return apostropheCheck and "strange thought process'" or "strange thought process"
			end
			local chosen = random(1, total)
			for i=1,#list do
				if chosen <= list[i].amount then
					local pattern = string_rep("[^|%(%)]+|", chosen - 1) .. "([^%(|%)]+)"
					local text = list[i].text:match(pattern) or "strange thought process"
					return apostropheCheck and ApostropheName(text) or text
				end
				chosen = chosen - list[i].amount
			end
		end)

		-- replace () options
		line = ExpandOptions(line)

		-- replace normal variables
			line = line:gsub("<(.-)>", function(variable)
				local apostropheCheck
				if variable:sub(-2) == "'s" then
					variable = variable:sub(1, -3)
					apostropheCheck = true
				end
				local name, arg = variable:match("([%w_]+)[:]*(.*)")
				local text = Variables:Lookup(name, arg) or "<"..variable..">"
				return apostropheCheck and ApostropheName(text) or text
			end)

		-- replace symbol variables
		line = line:gsub("(<[%w_]->)", function(variable)
			local info = Variables.symbols[variable]
			return info or variable
		end)

		-- fix the usage of "a" and "an"
		line = line:gsub("%f[%w](%s?)([aA][nN]?) (%w+)", function(before, an, word)
			local lowerWord = word:lower() or ""
			for i=1,#ArticleListA do
				if lowerWord:find(ArticleListA[i]) then
					return before .. (an:sub(1,1) == "A" and "A " or "a ") .. word
				end
			end
			for i=1,#ArticleListAn do
				if lowerWord:find(ArticleListAn[i]) then
					return before .. ((#an == 2 and an .. " ") or (an:sub(1,1) == "A" and "An " or "an ")) .. word
				end
			end
			return before .. (an:sub(1,1) == "A" and "A " or "a ") .. word
		end)

		-- redirect to another spell/event
		if lineChannel == "spell/event" then
			--[[
			reactionData is {
				hitType,
				Variables.casterName,
				Variables.targetName,
				actionType,
				force,
				forceMessage,
				redirectionSpellName or spellName,
				redirectionSpellId or spellId,
				redirectionFromEvent,
				redirectionCount
			}
			--]]
			if reactionData then
				reactionData[10] = reactionData[10] and (reactionData[10] + 1) or 1
				if reactionData[10] and reactionData[10] > 7 then
					DEFAULT_CHAT_FRAME:AddMessage("Reactions: Too many spell/event redirections! Last redirection to: " .. (reactionData[7] or "Unknown Name") .. " (ID " .. (reactionData[8] or "unknown") .. ")")
					break -- acts as continue
				end
				AttemptReaction(line, 0, reactionData[1], reactionData[2], reactionData[3], reactionData[4],
									 reactionData[5], reactionData[6], reactionData[7], reactionData[8],
									 reactionData[9], reactionData[10], lineTime)
			else
				AttemptReaction(line, 0, nil, UnitName("player"), nil, RS.Common.ActionType.HIT, nil, nil, nil, nil, nil, 1, lineTime)
			end
			break -- acts as continue
		end

		-- convert special channels to their actual channel
		if lineChannel ~= "chat command" and lineChannel ~= "print to chat" and lineChannel ~= "print to warning" and
			lineChannel ~= "force_message" and lineChannel ~= "say and yell" then
			if lineChannel == "group" then
				lineChannel = "chat command"
				local inInstance, instanceType = IsInInstance()
				if UnitInBattleground("player") then
					line = SLASH_BATTLEGROUND1 .. " " .. line
				elseif IsInRaid() then
					line = SLASH_RAID1 .. " " .. line
				elseif IsInGroup() then
					line = SLASH_PARTY1 .. " " .. line
				else
					break -- acts as continue
				end
			elseif lineChannel == "whisper" or lineChannel == "/w target" then
				if not Variables.targetName and not Variables.extraTargetName and not Variables.memberName then
					break -- acts as continue
				end
				lineChannel = "chat command"
				line = SLASH_WHISPER1 .. " " .. (Variables.extraTargetName or Variables.targetName or Variables.memberName) .. " " .. line
			elseif lineChannel == "/w caster" then
				if not Variables.casterName and not Variables.targetName then
					break -- acts as continue
				end
				lineChannel = "chat command"
				line = SLASH_WHISPER1 .. " " .. (Variables.casterName or Variables.targetName) .. " " .. line
			else
				break -- acts as continue
			end
		end

		local addonSettings = ReactionsSave

		-- shout mode
		if addonSettings.shoutMode then
			-- can't capitalize everything because parts of item/spell/etc links must be the proper case
			-- example link: "text |cff9d9d9d|Hitem:7073:0:0:0:0:0:0:0:80:0:0:0:0|h[Broken Fang]|h|r text"
			-- |c|r|h must be lowercase, except at |Hitem where |H must be capital and "item" must be lowercase
			-- so 1. make all uppercase, 2. make all "|<letter>" lowercase, 3. make "|h<word>:" be "|H<lowercase word>:"
			--
			-- This should work for all normal links and all intentionally messed up links except for where they remove
			-- the [] around the name and the first word of the name has : at the end, like: |Hitem:199:0:0:0:0:0:0:0:80:0:0:0:0|hSomething: bad name|h
			-- This can't be fixed by using "|H(%u+:%d)" because name links don't have a number after the : and I don't
			-- want to check the word after |H to see if it is item/name/enchant/spell/etc just for this one strange case
			line = line:upper():gsub("|[CRH]", string.lower):gsub("|h(%u+:)", function(w) return "|H" .. w:lower() end)
		else
			-- capitalize the first letter of some things
			local command = line:match("^(/%S)")
			if not command then
				line = line:gsub("^%l", string_upper)
			elseif capitalizeChannels[command:lower()] then
				line = line:gsub("^(/%S )(%l)", function(cmd, letter) return cmd..letter:upper() end)
			end
		end

		-- decide language to use
		local languageName = nil
		if language and language ~= "Common" and GetNumLanguages() > 1 then
			if language == "Racial" then
				if UnitFactionGroup("player") == "Horde" then
					languageName = GetLanguageByIndex(GetLanguageByIndex(1) == "Orcish" and 2 or 1)
				else
					languageName = GetLanguageByIndex(GetLanguageByIndex(1) == "Common" and 2 or 1)
				end
			else
				languageName = GetLanguageByIndex(random(1, GetNumLanguages()))
			end
		end

		--------------------------------------------------
		-- show/use the message
		--------------------------------------------------
		if lineChannel == "say and yell" then
			if lineTime > 0 then
				RS.Queue:AddFutureReaction(SLASH_SAY1.." "..line, "chat command", languageName, lineTime)
				RS.Queue:AddFutureReaction(SLASH_YELL1.." "..line, "chat command", languageName, lineTime)
			else
				Messages.UseMessage(SLASH_SAY1.." "..line, "chat command", languageName)
				Messages.UseMessage(SLASH_YELL1.." "..line, "chat command", languageName)
			end
		else
			if lineTime > 0 then
				RS.Queue:AddFutureReaction(line, lineChannel, languageName, lineTime)
			else
				Messages.UseMessage(line, lineChannel, languageName)
			end
		end
	until true
	end
end
